from jhomeassistant.types.units.base import BaseUnit


class SpeedUnit(BaseUnit):
    FOOT_PER_SECOND = "ft/s"
    KILOMETER_PER_HOUR = "km/h"
    KNOT = "kn"
    METER_PER_SECOND = "m/s"
    MILE_PER_HOUR = "mph"
    INCH_PER_DAY = "in/d"
    INCH_PER_HOUR = "in/h"
    INCH_PER_SECOND = "in/s"
    MILLIMETER_PER_DAY = "mm/d"
    MILLIMETER_PER_SECOND = "mm/s"



from __future__ import annotations

from typing import Union, List

from ..topic import TopicConfig
from .availability_item import AvailabilityItem
from jhomeassistant.setup_logging import get_logger
from jhomeassistant.types import AvailabilityMode
from jhomeassistant.helper import validate_topic
from jhomeassistant.helper.abbreviations import Abbreviation


logger = get_logger("Availability")


class Availability:
    def __init__(self, topic=None):
        self._mode = None
        self._items: List[AvailabilityItem] = []
        if topic is not None:
            self.add(topic)

    def __getitem__(self, key: Union[int, str]):
        if isinstance(key, int):
            return self._items[key]
        return next((i for i in self._items if i.topic == key), None)

    def __iter__(self):
        return iter(self._items)

    def remove(self, topic: str):
        """Remove an availability source by topic."""
        item = self[topic]
        if item is None:
            raise KeyError(f"Topic {topic!r} not found.")
        self._items.remove(item)
        logger.info(f"Removed topic={topic!r}")
        return self

    def deactivate(self):
        """Deactivate all availability sources."""
        self._items = []
        return self

    def add(self, topic: str, payload_available: str = TopicConfig.DEFAULT_AVAILABLE,
            payload_not_available: str = TopicConfig.DEFAULT_NOT_AVAILABLE, value_template: str | None = None):
        """Add a new availability source."""
        validate_topic(topic)
        if any(i.topic == topic for i in self._items):
            raise ValueError(f"Availability for topic {topic!r} already exists. Modify it via Availability[{topic!r}] or Availability[index].")

        self._items.append(AvailabilityItem(topic, payload_available, payload_not_available, value_template))
        logger.info(f"Added availability topic={topic!r}.")
        return self

    @property
    def mode(self):
        return self._mode if self._mode is not None else AvailabilityMode.LATEST

    @mode.setter
    def mode(self, mode: AvailabilityMode):
        self._mode = mode

    @property
    def active(self):
        return len(self._items) > 0

    def internal_merge(self, availability: Availability):
        """
        Merge availability parameter into self.
        """
        for item in availability:
            own_item = self[item.topic]
            has_item = own_item is not None
            if has_item:
                logger.warning(f"Duplicate availability topic during merge. Preferring own availability item: {own_item!r}")
            else:
                self._items.append(item)

    def internal_to_dict(self):
        result = {}
        if len(self._items) > 0:
            result[Abbreviation.AVAILABILITY] = [i.internal_to_dict() for i in self._items]
        if self._mode is not None:
            result[Abbreviation.AVAILABILITY_MODE] = self._mode

        return result



from .number_device_class import NumberDeviceClass
from .sensor_device_class import SensorDeviceClass
from .binary_sensor_device_class import BinarySensorDeviceClass
from .button_device_class import ButtonDeviceClass
from .cover_device_class import CoverDeviceClass
from .event_device_class import EventDeviceClass
from .valve_device_class import ValveDeviceClass
from .switch_device_class import SwitchDeviceClass
from .update_device_class import UpdateDeviceClass
from .humidifier_device_class import HumidifierDeviceClass
from .media_player_device_class import MediaPlayerDeviceClass



from jhomeassistant.types.units.base import BaseUnit


class EnergyUnit(BaseUnit):
    JOULE = "J"
    KILOJOULE = "kJ"
    MEGAJOULE = "MJ"
    GIGAJOULE = "GJ"
    MILLIWATT_HOUR = "mWh"
    WATT_HOUR = "Wh"
    KILOWATT_HOUR = "kWh"
    MEGAWATT_HOUR = "MWh"
    GIGAWATT_HOUR = "GWh"
    TERAWATT_HOUR = "TWh"
    CALORIE = "cal"
    KILOCALORIE = "kcal"
    MEGACALORIE = "Mcal"
    GIGACALORIE = "Gcal"



from __future__ import annotations

import re
from typing import List, Tuple, Any
from jhomeassistant.setup_logging import get_logger
from jhomeassistant.types.component import Component
from jhomeassistant.types.device_classes.base_device_class import BaseDeviceClass
from jhomeassistant.types.units import MonetaryUnit
from jhomeassistant.types.units.base import BaseUnit

logger = get_logger("validation")


def validate_non_empty_string(value: str | None, field: str, allow_none=False) -> str | None:
    """
    Validate that 'value' is a non-empty string.
    """
    if allow_none and value is None:
        return value
    if not isinstance(value, str):
        raise ValueError(f"{field} is invalid: value must be a string.")
    if value.strip() == "":
        raise ValueError(f"{field} is invalid: value must not be empty.")
    return value


def validate_discovery_prefix(value: str) -> str:
    """
    Validate a Home Assistant MQTT discovery prefix.

    Rules:
    - Non-empty UTF-8 string.
    - Must not start with '$' (avoid $SYS and system namespaces).
    - Must not contain MQTT wildcards '+' or '#'.
    - Must not contain the null character.
    - No leading or trailing whitespace.
    - No empty topic segments (reject '//' or leading/trailing '/').
    - Whitespace inside is rejected to avoid fragile topics.
    """
    validate_non_empty_string(value, "discovery_prefix")
    if value != value.strip():
        raise ValueError("discovery_prefix is invalid: leading or trailing whitespace is not allowed.")
    if value.startswith("$"):
        raise ValueError("discovery_prefix is invalid: must not start with '$'.")
    if "+" in value or "#" in value:
        raise ValueError("discovery_prefix is invalid: wildcards '+' and '#' are not allowed.")
    if "\x00" in value:
        raise ValueError("discovery_prefix is invalid: null character is not allowed.")
    if any(char.isspace() for char in value):
        raise ValueError("discovery_prefix is invalid: whitespace characters are not allowed.")
    if value.startswith("/") or value.endswith("/"):
        raise ValueError("discovery_prefix is invalid: must not start or end with '/'.")
    if "//" in value:
        raise ValueError("discovery_prefix is invalid: empty topic segments are not allowed (no '//').")

    # UTF-8 encodability check
    try:
        value.encode("utf-8")
    except UnicodeEncodeError:
        raise ValueError("discovery_prefix is invalid: must be UTF-8 encodable.")
    return value


# Not necessary :
def _validate_discovery_topic_id(value: str, id_name: str) -> str:
    """
    Validate a Node ID or Object ID for Home Assistant MQTT discovery.

    Rules for both:
    - Only letters, digits, underscore, hyphen: [A-Za-z0-9_-]+
    - No slashes, no spaces, no other symbols.
    - Non-empty.

    Returns the input value if valid.
    Raises ValidationError with a clear reason if invalid.
    """
    validate_non_empty_string(value, id_name)
    if not re.fullmatch(r"^[A-Za-z0-9_-]+$", value):
        raise ValueError(f"{id_name} is invalid: only letters, digits, underscore, and hyphen are allowed.")
    return value


"""def validate_object_id(value: str) -> str:
    return _validate_discovery_topic_id(value, "object_id")


def validate_node_id(value: str | None) -> str | None:
    return value if value is None else _validate_discovery_topic_id(value, "node_id")"""


def validate_topic(topic: str, strict_mode: bool = False) -> str:
    """
    Validate an MQTT topic name.

    Spec-enforced checks (always errors):
      - Non-empty UTF-8 string.
      - No null character ('\\x00').
      - UTF-8 length <= 65535 bytes.
      - No '+' or '#' (wildcards are for filters, not topic names).

    Best-practice checks (warnings by default; errors if strict_mode=True):
      - No leading/trailing whitespace.
      - No internal whitespace.
      - Do not start with '$' (reserved for broker/$SYS).
      - Avoid leading/trailing '/'.
      - Avoid empty levels ('//').

    Returns the topic if valid, else raises ValueError.
    """

    # ---- Spec-enforced ----
    validate_non_empty_string(topic, "topic")
    if "\x00" in topic:
        raise ValueError("topic is invalid: null character is not allowed.")

    try:
        encoded = topic.encode("utf-8")
    except UnicodeEncodeError:
        raise ValueError("topic is invalid: must be UTF-8 encodable.")

    if len(encoded) > 65535:
        raise ValueError("topic is invalid: UTF-8 length must be ≤ 65535 bytes.")
    if "+" in topic or "#" in topic:
        raise ValueError("topic is invalid: wildcards '+' and '#' are not allowed in topic names.")

    # ---- Best practices (warn or error) ----
    bp_issues = []

    if topic != topic.strip():
        bp_issues.append("leading or trailing whitespace is not recommended.")
    if any(c.isspace() for c in topic):
        bp_issues.append("whitespace inside topic is not recommended.")
    if topic.startswith("$"):
        bp_issues.append("topics starting with '$' are reserved for broker/system.")
    if topic.startswith("/") or topic.endswith("/"):
        bp_issues.append("avoid leading or trailing '/'.")
    if "//" in topic:
        bp_issues.append("avoid empty topic levels ('//').")

    if bp_issues:
        msg = "validate_topic best-practice issues: " + "; ".join(bp_issues)
        if strict_mode:
            logger.error(msg)
            raise ValueError("topic violates best practices: " + "; ".join(bp_issues))
        else:
            logger.warning(msg)

    return topic


def validate_icon(name: str) -> str:
    """
    Validate an MDI icon name like 'mdi:play-circle'.
    Returns the name typed as IconName if valid.
    Raises ValueError on invalid input.
    """
    if not isinstance(name, str) or not re.fullmatch(r"^mdi:[a-z0-9_\-]+$", name):
        raise ValueError("""Icon not valid.
                         Information: https://www.home-assistant.io/docs/frontend/icons/.
                         Allowed icons: https://pictogrammers.com/library/mdi/""")
    return name


def get_entity_spec_help_links(component: Component | None = None) -> List[str]:
    additional_help = {
        Component.BINARY_SENSOR: "https://www.home-assistant.io/integrations/binary_sensor/#device-class",
        Component.BUTTON: "https://www.home-assistant.io/integrations/button/#device-class",
        Component.COVER: "https://www.home-assistant.io/integrations/cover/#device-class",
        Component.EVENT: "https://www.home-assistant.io/integrations/event/#device-class",
        Component.HUMIDIFIER: "https://www.home-assistant.io/integrations/humidifier/#device-class",
        Component.MEDIA_PLAYER: "https://www.home-assistant.io/integrations/media_player/#device-class",
        Component.NUMBER: "https://www.home-assistant.io/integrations/number/#device-class",
        Component.SENSOR: "https://www.home-assistant.io/integrations/sensor#device-class",
        Component.SWITCH: "https://www.home-assistant.io/integrations/switch/#device-class",
        Component.UPDATE: "https://www.home-assistant.io/integrations/update/#device-class",
        Component.VALVE: "https://www.home-assistant.io/integrations/valve/#device-class"
    }
    help_links = ["https://www.home-assistant.io/integrations/homeassistant/#device-class"]
    if component in additional_help:
        help_links.append(additional_help[component])
    return help_links


"""def _validate_platform(platform: Component, allowed_config) -> Component:
    if platform not in allowed_config:
        allowed_platforms = sorted(allowed_config.keys())
        raise ValueError(f"Unsupported platform '{platform}'. Allowed: {allowed_platforms}. See {get_entity_spec_help_links()}")
    return platform"""


def validate_entity_specification(
    component: Component,
    device_class: BaseDeviceClass,
    unit: BaseUnit,
) -> Tuple[Component, Any, Any]:
    """
    Validate a Home Assistant entity specification.

    Returns the normalized (component, device_class, unit_of_measurement) tuple.

    Rules:
    - Platform must exist in allowed_config.
    - Device class must be allowed for the component.
    - Unit must be one of the allowed units for that device class.
      If a class lists [None], no unit is allowed.
      Special case: device_class 'monetary' accepts any ISO-4217 3-letter code.
    """

    if device_class not in component.device_class:
        raise ValueError(
            f"Invalid device_class '{device_class}' for component '{component}'. "
            f"Allowed: {component.device_class}. See {get_entity_spec_help_links(component)}"
        )

    if unit not in component.device_class.unit and type(unit) is not MonetaryUnit:
        raise ValueError(
            f"Invalid unit '{unit}' for component '{component}' and device_class '{device_class}'. "
            f"Allowed: {component.device_class.unit}. See {get_entity_spec_help_links(component)}"
        )

    return component, device_class, unit



from enum import StrEnum


class AvailabilityMode(StrEnum):
    """
    Defines how to derive availability when combining multiple inputs.

    - LATEST: Use the most recently updated source or snapshot when values conflict.
    - ALL:    Condition is true only if every input/source satisfies it (logical AND; set intersection).
    - ANY:    Condition is true if at least one input/source satisfies it (logical OR; set union).

    Guidance:
        Use ALL for strict consistency, ANY for higher coverage and fault tolerance,
        and LATEST when recency is the primary tiebreaker between conflicting values.
    """
    LATEST = "latest"  # Most recent source wins
    ALL = "all"        # All sources must agree
    ANY = "any"        # One agreeing source is enough



from jhomeassistant.types.units.base import BaseUnit


class CarbonMonoxideSensorUnit(BaseUnit):
    PARTS_PER_MILLION = "ppm"
    MICROGRAM_PER_CUBIC_METER = "µg/m³"
    MILLIGRAM_PER_CUBIC_METER = "mg/m³"


class CarbonMonoxideNumberUnit(BaseUnit):
    PARTS_PER_MILLION = "ppm"



from jhomeassistant.types.units.base import BaseUnit


class DataSizeUnit(BaseUnit):
    BIT = "bit"
    KILOBIT = "kbit"
    MEGABIT = "Mbit"
    GIGABIT = "Gbit"
    BYTE = "B"
    KILOBYTE = "kB"
    MEGABYTE = "MB"
    GIGABYTE = "GB"
    TERABYTE = "TB"
    PETABYTE = "PB"
    EXABYTE = "EB"
    ZETTABYTE = "ZB"
    YOTTABYTE = "YB"
    KIBIBYTE = "KiB"
    MEBIBYTE = "MiB"
    GIBIBYTE = "GiB"
    TEBIBYTE = "TiB"
    PEBIBYTE = "PiB"
    EXBIBYTE = "EiB"
    ZEBIBYTE = "ZiB"
    YOBIBYTE = "YiB"



from jhomeassistant.types.device_classes.base_device_class import BaseDeviceClass
from jhomeassistant.types.units import AbsoluteHumidityUnit, ApparentPowerUnit, AreaUnit, AtmosphericPressureUnit, \
    BatteryUnit, BloodGlucoseConcentrationUnit, CarbonDioxideUnit, CarbonMonoxideNumberUnit, CurrentUnit, DataRateUnit, \
    DataSizeUnit, DistanceUnit, DurationUnit, EnergyUnit, EnergyDistanceUnit, EnergyStorageUnit, FrequencyUnit, \
    GasUnit, HumidityUnit, IlluminanceUnit, IrradianceUnit, MoistureUnit, MonetaryUnit, NitrogenDioxideUnit, \
    NitrogenMonoxideUnit, NitrousOxideUnit, OzoneUnit, Pm1Unit, Pm25Unit, Pm4Unit, Pm10Unit, PowerUnit, \
    PowerFactorUnit, PrecipitationUnit, PrecipitationIntensityUnit, PressureNumberUnit, ReactiveEnergyUnit, \
    ReactivePowerUnit, SignalStrengthUnit, SoundPressureUnit, SpeedUnit, SulphurDioxideUnit, TemperatureUnit, \
    VolatileOrganicCompoundsUnit, VolatileOrganicCompoundsPartsUnit, VoltageUnit, VolumeUnit, VolumeFlowRateNumberUnit, \
    VolumeStorageUnit, WaterUnit, WeightUnit, WindDirectionUnit, WindSpeedUnit


class NumberDeviceClass(BaseDeviceClass):
    NONE = None
    ABSOLUTE_HUMIDITY = "absolute_humidity"
    APPARENT_POWER = "apparent_power"
    AQI = "aqi"
    AREA = "area"
    ATMOSPHERIC_PRESSURE = "atmospheric_pressure"
    BATTERY = "battery"
    BLOOD_GLUCOSE_CONCENTRATION = "blood_glucose_concentration"
    CARBON_DIOXIDE = "carbon_dioxide"
    CARBON_MONOXIDE = "carbon_monoxide"
    CURRENT = "current"
    DATA_RATE = "data_rate"
    DATA_SIZE = "data_size"
    DISTANCE = "distance"
    DURATION = "duration"
    ENERGY = "energy"
    ENERGY_DISTANCE = "energy_distance"
    ENERGY_STORAGE = "energy_storage"
    FREQUENCY = "frequency"
    GAS = "gas"
    HUMIDITY = "humidity"
    ILLUMINANCE = "illuminance"
    IRRADIANCE = "irradiance"
    MOISTURE = "moisture"
    MONETARY = "monetary"
    NITROGEN_DIOXIDE = "nitrogen_dioxide"
    NITROGEN_MONOXIDE = "nitrogen_monoxide"
    NITROUS_OXIDE = "nitrous_oxide"
    OZONE = "ozone"
    PH = "ph"
    PM1 = "pm1"
    PM25 = "pm25"
    PM4 = "pm4"
    PM10 = "pm10"
    POWER = "power"
    POWER_FACTOR = "power_factor"
    PRECIPITATION = "precipitation"
    PRECIPITATION_INTENSITY = "precipitation_intensity"
    PRESSURE = "pressure"
    REACTIVE_ENERGY = "reactive_energy"
    REACTIVE_POWER = "reactive_power"
    SIGNAL_STRENGTH = "signal_strength"
    SOUND_PRESSURE = "sound_pressure"
    SPEED = "speed"
    SULPHUR_DIOXIDE = "sulphur_dioxide"
    TEMPERATURE = "temperature"
    VOLATILE_ORGANIC_COMPOUNDS = "volatile_organic_compounds"
    VOLATILE_ORGANIC_COMPOUNDS_PARTS = "volatile_organic_compounds_parts"
    VOLTAGE = "voltage"
    VOLUME = "volume"
    VOLUME_FLOW_RATE = "volume_flow_rate"
    VOLUME_STORAGE = "volume_storage"
    WATER = "water"
    WEIGHT = "weight"
    WIND_DIRECTION = "wind_direction"
    WIND_SPEED = "wind_speed"

    @property
    def unit(self):
        return {
            NumberDeviceClass.ABSOLUTE_HUMIDITY: AbsoluteHumidityUnit,
            NumberDeviceClass.APPARENT_POWER: ApparentPowerUnit,
            NumberDeviceClass.AREA: AreaUnit,
            NumberDeviceClass.ATMOSPHERIC_PRESSURE: AtmosphericPressureUnit,
            NumberDeviceClass.BATTERY: BatteryUnit,
            NumberDeviceClass.BLOOD_GLUCOSE_CONCENTRATION: BloodGlucoseConcentrationUnit,
            NumberDeviceClass.CARBON_DIOXIDE: CarbonDioxideUnit,
            NumberDeviceClass.CARBON_MONOXIDE: CarbonMonoxideNumberUnit,
            NumberDeviceClass.CURRENT: CurrentUnit,
            NumberDeviceClass.DATA_RATE: DataRateUnit,
            NumberDeviceClass.DATA_SIZE: DataSizeUnit,
            NumberDeviceClass.DISTANCE: DistanceUnit,
            NumberDeviceClass.DURATION: DurationUnit,
            NumberDeviceClass.ENERGY: EnergyUnit,
            NumberDeviceClass.ENERGY_DISTANCE: EnergyDistanceUnit,
            NumberDeviceClass.ENERGY_STORAGE: EnergyStorageUnit,
            NumberDeviceClass.FREQUENCY: FrequencyUnit,
            NumberDeviceClass.GAS: GasUnit,
            NumberDeviceClass.HUMIDITY: HumidityUnit,
            NumberDeviceClass.ILLUMINANCE: IlluminanceUnit,
            NumberDeviceClass.IRRADIANCE: IrradianceUnit,
            NumberDeviceClass.MOISTURE: MoistureUnit,
            NumberDeviceClass.MONETARY: MonetaryUnit,
            NumberDeviceClass.NITROGEN_DIOXIDE: NitrogenDioxideUnit,
            NumberDeviceClass.NITROGEN_MONOXIDE: NitrogenMonoxideUnit,
            NumberDeviceClass.NITROUS_OXIDE: NitrousOxideUnit,
            NumberDeviceClass.OZONE: OzoneUnit,
            NumberDeviceClass.PM1: Pm1Unit,
            NumberDeviceClass.PM25: Pm25Unit,
            NumberDeviceClass.PM4: Pm4Unit,
            NumberDeviceClass.PM10: Pm10Unit,
            NumberDeviceClass.POWER: PowerUnit,
            NumberDeviceClass.POWER_FACTOR: PowerFactorUnit,
            NumberDeviceClass.PRECIPITATION: PrecipitationUnit,
            NumberDeviceClass.PRECIPITATION_INTENSITY: PrecipitationIntensityUnit,
            NumberDeviceClass.PRESSURE: PressureNumberUnit,
            NumberDeviceClass.REACTIVE_ENERGY: ReactiveEnergyUnit,
            NumberDeviceClass.REACTIVE_POWER: ReactivePowerUnit,
            NumberDeviceClass.SIGNAL_STRENGTH: SignalStrengthUnit,
            NumberDeviceClass.SOUND_PRESSURE: SoundPressureUnit,
            NumberDeviceClass.SPEED: SpeedUnit,
            NumberDeviceClass.SULPHUR_DIOXIDE: SulphurDioxideUnit,
            NumberDeviceClass.TEMPERATURE: TemperatureUnit,
            NumberDeviceClass.VOLATILE_ORGANIC_COMPOUNDS: VolatileOrganicCompoundsUnit,
            NumberDeviceClass.VOLATILE_ORGANIC_COMPOUNDS_PARTS: VolatileOrganicCompoundsPartsUnit,
            NumberDeviceClass.VOLTAGE: VoltageUnit,
            NumberDeviceClass.VOLUME: VolumeUnit,
            NumberDeviceClass.VOLUME_FLOW_RATE: VolumeFlowRateNumberUnit,
            NumberDeviceClass.VOLUME_STORAGE: VolumeStorageUnit,
            NumberDeviceClass.WATER: WaterUnit,
            NumberDeviceClass.WEIGHT: WeightUnit,
            NumberDeviceClass.WIND_DIRECTION: WindDirectionUnit,
            NumberDeviceClass.WIND_SPEED: WindSpeedUnit
        }.get(self, super().unit)



from jhomeassistant.types.units.base import BaseUnit


class ConductivityUnit(BaseUnit):
    SIEMENS_PER_CENTIMETER = "S/cm"
    MILLISIEMENS_PER_CENTIMETER = "mS/cm"
    MICROSIEMENS_PER_CENTIMETER = "µS/cm"



from jhomeassistant.types.units.base import BaseUnit


class EnergyDistanceUnit(BaseUnit):
    KILOWATT_HOUR_PER_ONE_HUNDRED_KILOMETERS = "kWh/100km"
    WATT_HOUR_PER_KILOMETER = "Wh/km"
    MILE_PER_KILOWATT_HOUR = "mi/kWh"
    KILOMETER_PER_KILOWATT_HOUR = "km/kWh"



from __future__ import annotations
from typing import List

from jmqtt import QualityOfService as QoS
from jhomeassistant.features import Availability
from jhomeassistant.helper import collect_ha_device_facts, build_identifier, get_default_entity_id, \
    validate_non_empty_string
from jhomeassistant.entities import HomeAssistantEntityBase
from jhomeassistant.helper.abbreviations import DeviceAbbreviation, Abbreviation
from jhomeassistant.setup_logging import get_logger as _get_logger

logger = _get_logger("HomeAssistantDevice")


def get_identifier_default(serial_number, connections, prevent_device_merge=False):
    identifiers = []
    if not prevent_device_merge:
        if serial_number is not None:
            identifiers.append(build_identifier(serial_number))

        # Priority: mac > bluetooth; within type pick smallest value for stability
        for _, value in sorted(
                connections,
                key=lambda x: ((0 if x[0] == "mac" else 1 if x[0] == "bluetooth" else 2), x[1])
        ):
            identifiers.append(build_identifier(value))
            break
    return identifiers


class HomeAssistantDevice:
    def __init__(self, name: str, prevent_device_merge=False):
        validate_non_empty_string(name, "Device 'name'")

        self.name = name
        self.serial_number, self.connections = collect_ha_device_facts(prevent_device_merge)
        self.identifiers: List[str] = get_identifier_default(self.serial_number, self.connections, prevent_device_merge)

        if not self.identifiers:
            logger.error("No device identifiers could be derived. Set at least one identifier explicitly before discovery.")
        else:
            logger.info(f"Using device identifiers for {self.name}: {self.identifiers}")

        self.manufacturer: str | None = None
        self.model: str | None = None
        self.model_id: str | None = None
        self.hw_version: str | None = None
        self.sw_version: str | None = None
        self.via_device: str | None = None
        self.configuration_url: str | None = None

        self.qos: QoS | None = None
        self.encoding: str | None = None

        self.availability = Availability()
        self._entities: List[HomeAssistantEntityBase] = []

    def _to_dict(self):
        result = {
            DeviceAbbreviation.NAME: self.name,
            DeviceAbbreviation.IDS: self.identifiers,
        }

        if len(self.connections) > 0:
            result[DeviceAbbreviation.CONNECTIONS] = self.connections

        for abbr, value in (
                (DeviceAbbreviation.SERIAL_NUMBER, self.serial_number),
                (DeviceAbbreviation.MANUFACTURER, self.manufacturer),
                (DeviceAbbreviation.MODEL, self.model),
                (DeviceAbbreviation.MODEL_ID, self.model_id),
                (DeviceAbbreviation.HW, self.hw_version),
                (DeviceAbbreviation.SW, self.sw_version),
                (DeviceAbbreviation.VIA_DEVICE, self.via_device),
                (DeviceAbbreviation.URL, self.configuration_url),
        ):
            if value is not None:
                result[abbr] = value

        return {
            Abbreviation.DEVICE: result
        }

    @property
    def unique_id(self):
        return build_identifier(self.identifiers[0], 32, self.name)

    @property
    def entities(self) -> List[HomeAssistantEntityBase]:
        return self._entities

    def add_entities(self, *entities: HomeAssistantEntityBase) -> HomeAssistantDevice:
        self._entities.extend(entities)
        logger.info(f"Added {len(entities)} entities to device {self.name}. Total={len(self._entities),}")
        return self

    def internal_discovery(self, discovery_prefix):
        if len(self.identifiers) == 0:
            raise ValueError("At least one identifier must be specified. Use HomeAssistantDevice.identifiers to set one.")

        discovery_topic = f"{discovery_prefix}/device/{self.unique_id}/config"
        include_root_availability = not all(e.availability.active for e in self._entities)

        discovery_payload = {
            **self._to_dict(),
            Abbreviation.COMPONENTS: {},
            **(self.availability.internal_to_dict() if include_root_availability else {}),
        }

        if self.qos is not None:
            discovery_payload["qos"] = self.qos.value
        if self.encoding is not None:
            discovery_payload[Abbreviation.ENCODING] = self.encoding

        entity_ids: set[str] = set()
        for entity in self._entities:
            unique_id = f"{self.unique_id}-{entity.identifier}"
            default_entity_id = get_default_entity_id(entity.platform, self.name, entity.name)

            if unique_id in entity_ids:
                raise ValueError(f"Duplicate entity identifier({unique_id}) within device({self.name}). " +
                                 f"Entity names must yield unique identifiers per device. Either update device.identifiers[0] or entity.name")
            entity_ids.add(unique_id)

            if entity.availability.active:
                entity.availability.internal_merge(self.availability)

            discovery_payload[Abbreviation.COMPONENTS][unique_id] = {
                Abbreviation.UNIQUE_ID: unique_id,
                Abbreviation.DEFAULT_ENTITY_ID: default_entity_id,
                **entity.internal_discovery_payload
            }

        return discovery_topic, discovery_payload



from .abbriviation_resolver import resolve_abbreviation
from .device_abbreviation_enum import DeviceAbbreviation
from .origin_abbreviation_enum import OriginAbbreviation
from .abbreviation_enum import Abbreviation



from jhomeassistant.types.device_classes.base_device_class import BaseDeviceClass


class CoverDeviceClass(BaseDeviceClass):
    NONE = None
    AWNING = "awning"
    BLIND = "blind"
    CURTAIN = "curtain"
    DAMPER = "damper"
    DOOR = "door"
    GARAGE = "garage"
    GATE = "gate"
    SHADE = "shade"
    SHUTTER = "shutter"
    WINDOW = "window"



from jhomeassistant.types.units.base import BaseUnit


class PressureSensorUnit(BaseUnit):
    CENTIBAR = "cbar"
    BAR = "bar"
    MILLIBAR = "mbar"
    PASCAL = "Pa"
    HECTOPASCAL = "hPa"
    KILOPASCAL = "kPa"
    INCH_OF_MERCURY = "inHg"
    MILLIMETER_OF_MERCURY = "mmHg"
    POUND_PER_SQUARE_INCH = "psi"
    INCH_OF_WATER = "inH₂O"
    MILLIPASCAL = "mPa"


class PressureNumberUnit(BaseUnit):
    CENTIBAR = "cbar"
    BAR = "bar"
    MILLIBAR = "mbar"
    PASCAL = "Pa"
    HECTOPASCAL = "hPa"
    KILOPASCAL = "kPa"
    INCH_OF_MERCURY = "inHg"
    MILLIMETER_OF_MERCURY = "mmHg"
    POUND_PER_SQUARE_INCH = "psi"
    INCH_OF_WATER = "inH₂O"



from jhomeassistant.types.units.base import BaseUnit


class VolumeStorageUnit(BaseUnit):
    LITRE = "L"
    MILLILITRE = "mL"
    GALLON = "gal"
    CUBIC_FOOT = "ft³"
    CUBIC_METER = "m³"
    CENTUM_CUBIC_FEET = "CCF"
    THOUSAND_CUBIC_FEET = "MCF"
    FLUID_OUNCE = "fl. oz."



from jhomeassistant.types.device_classes.base_device_class import BaseDeviceClass
from jhomeassistant.types.units import (
    AbsoluteHumidityUnit,
    ApparentPowerUnit,
    AreaUnit,
    AtmosphericPressureUnit,
    BatteryUnit,
    BloodGlucoseConcentrationUnit,
    CarbonDioxideUnit,
    CarbonMonoxideSensorUnit,
    ConductivityUnit,
    CurrentUnit,
    DataRateUnit,
    DataSizeUnit,
    DistanceUnit,
    DurationUnit,
    EnergyUnit,
    EnergyDistanceUnit,
    EnergyStorageUnit,
    FrequencyUnit,
    GasUnit,
    HumidityUnit,
    IlluminanceUnit,
    IrradianceUnit,
    MoistureUnit,
    MonetaryUnit,
    NitrogenDioxideUnit,
    NitrogenMonoxideUnit,
    NitrousOxideUnit,
    OzoneUnit,
    Pm1Unit,
    Pm25Unit,
    Pm4Unit,
    Pm10Unit,
    PowerUnit,
    PowerFactorUnit,
    PrecipitationUnit,
    PrecipitationIntensityUnit,
    PressureSensorUnit,
    ReactiveEnergyUnit,
    ReactivePowerUnit,
    SignalStrengthUnit,
    SoundPressureUnit,
    SpeedUnit,
    SulphurDioxideUnit,
    TemperatureUnit,
    TemperatureDeltaUnit,
    VolatileOrganicCompoundsUnit,
    VolatileOrganicCompoundsPartsUnit,
    VoltageUnit,
    VolumeUnit,
    VolumeFlowRateSensorUnit,
    VolumeStorageUnit,
    WaterUnit,
    WeightUnit,
    WindDirectionUnit,
    WindSpeedUnit,
)


class SensorDeviceClass(BaseDeviceClass):
    NONE = None
    ABSOLUTE_HUMIDITY = "absolute_humidity"
    APPARENT_POWER = "apparent_power"
    AQI = "aqi"
    AREA = "area"
    ATMOSPHERIC_PRESSURE = "atmospheric_pressure"
    BATTERY = "battery"
    BLOOD_GLUCOSE_CONCENTRATION = "blood_glucose_concentration"
    CARBON_DIOXIDE = "carbon_dioxide"
    CARBON_MONOXIDE = "carbon_monoxide"
    CURRENT = "current"
    DATA_RATE = "data_rate"
    DATA_SIZE = "data_size"
    DISTANCE = "distance"
    DURATION = "duration"
    ENERGY = "energy"
    ENERGY_DISTANCE = "energy_distance"
    ENERGY_STORAGE = "energy_storage"
    FREQUENCY = "frequency"
    GAS = "gas"
    HUMIDITY = "humidity"
    ILLUMINANCE = "illuminance"
    IRRADIANCE = "irradiance"
    MOISTURE = "moisture"
    MONETARY = "monetary"
    NITROGEN_DIOXIDE = "nitrogen_dioxide"
    NITROGEN_MONOXIDE = "nitrogen_monoxide"
    NITROUS_OXIDE = "nitrous_oxide"
    OZONE = "ozone"
    PH = "ph"
    PM1 = "pm1"
    PM25 = "pm25"
    PM4 = "pm4"
    PM10 = "pm10"
    POWER = "power"
    POWER_FACTOR = "power_factor"
    PRECIPITATION = "precipitation"
    PRECIPITATION_INTENSITY = "precipitation_intensity"
    PRESSURE = "pressure"
    REACTIVE_ENERGY = "reactive_energy"
    REACTIVE_POWER = "reactive_power"
    SIGNAL_STRENGTH = "signal_strength"
    SOUND_PRESSURE = "sound_pressure"
    SPEED = "speed"
    SULPHUR_DIOXIDE = "sulphur_dioxide"
    TEMPERATURE = "temperature"
    VOLATILE_ORGANIC_COMPOUNDS = "volatile_organic_compounds"
    VOLATILE_ORGANIC_COMPOUNDS_PARTS = "volatile_organic_compounds_parts"
    VOLTAGE = "voltage"
    VOLUME = "volume"
    VOLUME_FLOW_RATE = "volume_flow_rate"
    VOLUME_STORAGE = "volume_storage"
    WATER = "water"
    WEIGHT = "weight"
    WIND_DIRECTION = "wind_direction"
    WIND_SPEED = "wind_speed"
    CONDUCTIVITY = "conductivity"
    DATE = "date"
    ENUM = "enum"
    TEMPERATURE_DELTA = "temperature_delta"
    TIMESTAMP = "timestamp"

    @property
    def unit(self):
        return {
            SensorDeviceClass.ABSOLUTE_HUMIDITY: AbsoluteHumidityUnit,
            SensorDeviceClass.APPARENT_POWER: ApparentPowerUnit,
            SensorDeviceClass.AREA: AreaUnit,
            SensorDeviceClass.ATMOSPHERIC_PRESSURE: AtmosphericPressureUnit,
            SensorDeviceClass.BATTERY: BatteryUnit,
            SensorDeviceClass.BLOOD_GLUCOSE_CONCENTRATION: BloodGlucoseConcentrationUnit,
            SensorDeviceClass.CARBON_DIOXIDE: CarbonDioxideUnit,
            SensorDeviceClass.CURRENT: CurrentUnit,
            SensorDeviceClass.DATA_RATE: DataRateUnit,
            SensorDeviceClass.DATA_SIZE: DataSizeUnit,
            SensorDeviceClass.DISTANCE: DistanceUnit,
            SensorDeviceClass.DURATION: DurationUnit,
            SensorDeviceClass.ENERGY: EnergyUnit,
            SensorDeviceClass.ENERGY_DISTANCE: EnergyDistanceUnit,
            SensorDeviceClass.ENERGY_STORAGE: EnergyStorageUnit,
            SensorDeviceClass.FREQUENCY: FrequencyUnit,
            SensorDeviceClass.GAS: GasUnit,
            SensorDeviceClass.HUMIDITY: HumidityUnit,
            SensorDeviceClass.ILLUMINANCE: IlluminanceUnit,
            SensorDeviceClass.IRRADIANCE: IrradianceUnit,
            SensorDeviceClass.MOISTURE: MoistureUnit,
            SensorDeviceClass.MONETARY: MonetaryUnit,
            SensorDeviceClass.NITROGEN_DIOXIDE: NitrogenDioxideUnit,
            SensorDeviceClass.NITROGEN_MONOXIDE: NitrogenMonoxideUnit,
            SensorDeviceClass.NITROUS_OXIDE: NitrousOxideUnit,
            SensorDeviceClass.OZONE: OzoneUnit,
            SensorDeviceClass.PM1: Pm1Unit,
            SensorDeviceClass.PM25: Pm25Unit,
            SensorDeviceClass.PM4: Pm4Unit,
            SensorDeviceClass.PM10: Pm10Unit,
            SensorDeviceClass.POWER: PowerUnit,
            SensorDeviceClass.POWER_FACTOR: PowerFactorUnit,
            SensorDeviceClass.PRECIPITATION: PrecipitationUnit,
            SensorDeviceClass.PRECIPITATION_INTENSITY: PrecipitationIntensityUnit,
            SensorDeviceClass.REACTIVE_ENERGY: ReactiveEnergyUnit,
            SensorDeviceClass.REACTIVE_POWER: ReactivePowerUnit,
            SensorDeviceClass.SIGNAL_STRENGTH: SignalStrengthUnit,
            SensorDeviceClass.SOUND_PRESSURE: SoundPressureUnit,
            SensorDeviceClass.SPEED: SpeedUnit,
            SensorDeviceClass.SULPHUR_DIOXIDE: SulphurDioxideUnit,
            SensorDeviceClass.TEMPERATURE: TemperatureUnit,
            SensorDeviceClass.VOLATILE_ORGANIC_COMPOUNDS: VolatileOrganicCompoundsUnit,
            SensorDeviceClass.VOLATILE_ORGANIC_COMPOUNDS_PARTS: VolatileOrganicCompoundsPartsUnit,
            SensorDeviceClass.VOLTAGE: VoltageUnit,
            SensorDeviceClass.VOLUME: VolumeUnit,
            SensorDeviceClass.VOLUME_STORAGE: VolumeStorageUnit,
            SensorDeviceClass.WATER: WaterUnit,
            SensorDeviceClass.WEIGHT: WeightUnit,
            SensorDeviceClass.WIND_DIRECTION: WindDirectionUnit,
            SensorDeviceClass.WIND_SPEED: WindSpeedUnit,
            SensorDeviceClass.CARBON_MONOXIDE: CarbonMonoxideSensorUnit,
            SensorDeviceClass.CONDUCTIVITY: ConductivityUnit,
            SensorDeviceClass.PRESSURE: PressureSensorUnit,
            SensorDeviceClass.VOLUME_FLOW_RATE: VolumeFlowRateSensorUnit,
            SensorDeviceClass.TEMPERATURE_DELTA: TemperatureDeltaUnit,
        }.get(self, super().unit)



class Schedule:
    def __init__(self, interval_sec: float, func):
        self._func = func
        self._next_timestamp = 0.0
        self._interval = max(0.0, interval_sec)

    def run(self, now: float, connection):
        if now >= self._next_timestamp:
            self._func(connection)
            self._next_timestamp = now + self._interval



from jhomeassistant.types.device_classes.base_device_class import BaseDeviceClass


class EventDeviceClass(BaseDeviceClass):
    NONE = None
    BUTTON = "button"
    DOORBELL = "doorbell"
    MOTION = "motion"



from jhomeassistant.types.units.base import BaseUnit


class WeightUnit(BaseUnit):
    KILOGRAM = "kg"
    GRAM = "g"
    MILLIGRAM = "mg"
    MICROGRAM = "µg"
    OUNCE = "oz"
    POUND = "lb"
    STONE = "st"



from jhomeassistant.types.units.base import BaseUnit


class AreaUnit(BaseUnit):
    SQUARE_METER = "m²"
    SQUARE_CENTIMETER = "cm²"
    SQUARE_KILOMETER = "km²"
    SQUARE_MILLIMETER = "mm²"
    SQUARE_INCH = "in²"
    SQUARE_FOOT = "ft²"
    SQUARE_YARD = "yd²"
    SQUARE_MILE = "mi²"
    ACRE = "ac"
    HECTARE = "ha"



from enum import Enum


class DeviceAbbreviation(Enum):
    NAME = ("name", "name")
    IDS = ("identifiers", "ids")
    CONNECTIONS = ("connections", "cns")
    SERIAL_NUMBER = ("serial_number", "sn")
    MANUFACTURER = ("manufacturer", "mf")
    MODEL = ("model", "mdl")
    MODEL_ID = ("model_id", "mdl_id")
    HW = ("hw_version", "hw")
    SW = ("sw_version", "sw")
    VIA_DEVICE = ("via_device", "via_device")
    URL = ("configuration_url", "cu")
    SUGGESTED_AREA = ("suggested_area", "sa")



from enum import Enum

from jhomeassistant.types.device_classes import SensorDeviceClass, NumberDeviceClass, BinarySensorDeviceClass, \
    ButtonDeviceClass, CoverDeviceClass, EventDeviceClass, ValveDeviceClass, UpdateDeviceClass, SwitchDeviceClass, \
    MediaPlayerDeviceClass, HumidifierDeviceClass


class Component(Enum):
    BINARY_SENSOR = "binary_sensor"
    BUTTON = "button"
    COVER = "cover"
    EVENT = "event"
    HUMIDIFIER = "humidifier"
    MEDIA_PLAYER = "media_player"
    NUMBER = "number"
    SENSOR = "sensor"
    SWITCH = "switch"
    UPDATE = "update"
    VALVE = "valve"

    @property
    def device_class(self):
        return {
            Component.SENSOR: SensorDeviceClass,
            Component.NUMBER: NumberDeviceClass,
            Component.BINARY_SENSOR: BinarySensorDeviceClass,
            Component.BUTTON: ButtonDeviceClass,
            Component.COVER: CoverDeviceClass,
            Component.EVENT: EventDeviceClass,
            Component.HUMIDIFIER: HumidifierDeviceClass,
            Component.MEDIA_PLAYER: MediaPlayerDeviceClass,
            Component.SWITCH: SwitchDeviceClass,
            Component.UPDATE: UpdateDeviceClass,
            Component.VALVE: ValveDeviceClass,
        }[self]



from jhomeassistant.types.units.base import BaseUnit


class DistanceUnit(BaseUnit):
    KILOMETER = "km"
    METER = "m"
    CENTIMETER = "cm"
    MILLIMETER = "mm"
    MILE = "mi"
    NAUTICAL_MILE = "nmi"
    YARD = "yd"
    INCH = "in"



from __future__ import annotations
from typing import Optional, Set, Tuple, List

Connection = Tuple[str, str]
CONN_MAC = "mac"
CONN_BT = "bluetooth"


def collect_ha_device_facts(prevent_merge: bool = False) -> tuple[Optional[str], List[Connection]]:
    serial: Optional[str] = None
    connections: Set[Connection] = set()
    runtime = _detect_runtime()

    # Serial (best-effort)
    try:
        if runtime == "MicroPython":
            serial = _serial_mpy()
        elif runtime == "Linux":
            serial = _serial_linux()
        elif runtime == "Darwin":
            serial = _serial_macos()
        elif runtime == "Windows":
            serial = _serial_windows()
    except Exception:
        pass

    # Connections (skip entirely if prevent_merge)
    if not prevent_merge:
        try:
            if runtime == "MicroPython":
                connections |= _conns_mpy()
            elif runtime == "Linux":
                connections |= _conns_linux()
            elif runtime == "Darwin":
                connections |= _conns_macos()
            elif runtime == "Windows":
                connections |= _conns_windows()
        except Exception:
            pass

    return serial, _normalize_connections(connections)


# ---------- runtime detection ----------

def _detect_runtime() -> str:
    try:
        import sys
        name = getattr(getattr(sys, "implementation", None), "name", "")
        if str(name).lower() == "micropython":
            return "MicroPython"
    except Exception:
        pass
    try:
        import platform
        return platform.system() or "Unknown"
    except Exception:
        return "Unknown"


# ---------- normalization ----------
def _normalize_mac(s: str) -> Optional[str]:
    if not isinstance(s, str):
        return None
    raw = "".join(c for c in s if c.isalnum()).lower()
    if len(raw) != 12 or any(c not in "0123456789abcdef" for c in raw):
        return None
    return ":".join(raw[i:i+2] for i in range(0, 12, 2))


def _is_global_mac(mac: str) -> bool:
    try:
        first = int(mac.split(":", 1)[0], 16)
        return not (first & 0x01) and not (first & 0x02)
    except Exception:
        return False


def _normalize_connections(conns: Set[Connection]) -> List[Connection]:
    out: Set[Connection] = set()
    for k, v in conns:
        mac = _normalize_mac(v or "")
        if not k or not mac:
            continue
        if mac in ("00:00:00:00:00:00", "ff:ff:ff:ff:ff:ff"):
            continue
        if k == CONN_MAC:
            if _is_global_mac(mac):           # ignore virtual/LAA network MACs
                out.add((CONN_MAC, mac))
        elif k == CONN_BT:
            out.add((CONN_BT, mac))           # accept BT even if locally administered
    return list(out)


def _run_cmd(cmd: list[str]) -> str:
    try:
        import subprocess  # nosec
        return subprocess.check_output(cmd, text=True, stderr=subprocess.DEVNULL)
    except Exception:
        return ""


# ---------- MicroPython ----------
def _conns_mpy() -> Set[Connection]:
    out: Set[Connection] = set()
    try:
        import binascii  # type: ignore
        try:
            import network  # type: ignore
            for name in ("STA_IF", "AP_IF"):
                try:
                    idx = getattr(network, name)
                    wlan = network.WLAN(idx)
                    mac_bytes = wlan.config("mac")
                    if mac_bytes:
                        mac = _normalize_mac(binascii.hexlify(mac_bytes).decode())
                        if mac and _is_global_mac(mac):
                            out.add((CONN_MAC, mac))
                except Exception:
                    continue
        except Exception:
            pass
    except Exception:
        pass
    return out


def _serial_mpy() -> Optional[str]:
    try:
        import binascii  # type: ignore
        import machine   # type: ignore
        if hasattr(machine, "unique_id"):
            uid = machine.unique_id()
            if isinstance(uid, (bytes, bytearray)):
                return binascii.hexlify(uid).decode()
            return str(uid)
    except Exception:
        pass
    return None


# ---------- Linux ----------
def _conns_linux() -> Set[Connection]:
    out: Set[Connection] = set()
    try:
        from pathlib import Path
        for p in Path("/sys/class/net").iterdir():
            try:
                addr = (p / "address").read_text().strip()
            except Exception:
                continue
            mac = _normalize_mac(addr)
            if mac and _is_global_mac(mac):
                out.add((CONN_MAC, mac))
    except Exception:
        pass
    try:
        from pathlib import Path
        # allow Bluetooth only if controller reports a public address
        info = _run_cmd(["btmgmt", "info"]).lower()
        is_public = "public address" in info

        root = Path("/sys/class/bluetooth")
        if root.exists():
            for ctrl in root.iterdir():
                try:
                    addr = (ctrl / "address").read_text().strip()
                    mac = _normalize_mac(addr)
                    if mac and is_public:
                        out.add((CONN_BT, mac))
                except Exception:
                    continue
        else:
            res = _run_cmd(["bluetoothctl", "show"])
            for line in res.splitlines():
                parts = line.strip().split()
                if parts[:1] == ["Controller"] and len(parts) >= 2:
                    mac = _normalize_mac(parts[1])
                    if mac and is_public:
                        out.add((CONN_BT, mac))
    except Exception:
        pass
    return out


def _serial_linux() -> Optional[str]:
    for path in (
        "/sys/class/dmi/id/product_serial",
        "/sys/firmware/devicetree/base/serial-number",
        "/proc/device-tree/serial-number",
    ):
        try:
            with open(path, "r", encoding="utf-8", errors="ignore") as f:
                s = f.read().strip().strip("\x00")
            if s:
                return s
        except Exception:
            continue
    try:
        with open("/proc/cpuinfo", "r", encoding="utf-8", errors="ignore") as f:
            for line in f:
                if line.lower().startswith("serial"):
                    s = line.split(":", 1)[1].strip()
                    if s:
                        return s
    except Exception:
        pass
    return None


# ---------- macOS ----------
def _conns_macos() -> Set[Connection]:
    out: Set[Connection] = set()
    try:
        res = _run_cmd(["networksetup", "-listallhardwareports"])
        for line in res.splitlines():
            line = line.strip()
            if line.lower().startswith("ethernet address"):
                mac = _normalize_mac(line.split(":", 1)[1].strip())
                if mac and _is_global_mac(mac):
                    out.add((CONN_MAC, mac))
    except Exception:
        pass
    try:
        res = _run_cmd(["system_profiler", "SPBluetoothDataType"])
        for line in res.splitlines():
            line = line.strip()
            if line.lower().startswith("address:"):
                mac = _normalize_mac(line.split(":", 1)[1].strip())
                # accept BT only if it looks globally administered (public)
                if mac and _is_global_mac(mac):
                    out.add((CONN_BT, mac))
                    break
    except Exception:
        pass
    return out


def _serial_macos() -> Optional[str]:
    try:
        res = _run_cmd(["ioreg", "-rd1", "-c", "IOPlatformExpertDevice"])
        for line in res.splitlines():
            if "IOPlatformSerialNumber" in line:
                s = line.split("=")[1].strip().strip('"')
                if s:
                    return s
    except Exception:
        pass
    return None


# ---------- Windows ----------
def _conns_windows() -> Set[Connection]:
    out: Set[Connection] = set()
    try:
        res = _run_cmd(["getmac", "/v", "/fo", "csv"])
        for line in res.splitlines():
            if "," not in line:
                continue
            for cell in [c.strip().strip('"') for c in line.split(",")]:
                mac = _normalize_mac(cell)
                if mac and _is_global_mac(mac):
                    out.add((CONN_MAC, mac))
    except Exception:
        pass
    if not any(k == CONN_BT for k, _ in out):
        try:
            res = _run_cmd(["wmic", "nic", "get", "Name,MACAddress", "/format:csv"])
            for row in res.splitlines():
                cols = [c.strip() for c in row.split(",")]
                if len(cols) >= 3 and "bluetooth" in cols[2].lower():
                    mac = _normalize_mac(cols[1])
                    # accept BT only if it looks globally administered (public)
                    if mac and _is_global_mac(mac):
                        out.add((CONN_BT, mac))
        except Exception:
            pass
    return out


def _serial_windows() -> Optional[str]:
    try:
        res = _run_cmd(["powershell", "-NoProfile", "-Command", "(Get-CimInstance Win32_BIOS).SerialNumber"])
        s = res.strip()
        if s:
            return s
    except Exception:
        pass
    try:
        res = _run_cmd(["wmic", "bios", "get", "serialnumber"])
        parts = [l.strip() for l in res.splitlines() if l.strip()]
        if len(parts) > 1:
            return parts[1]
    except Exception:
        pass
    return None



from __future__ import annotations

from jhomeassistant.helper import validate_topic, validate_non_empty_string


class TopicConfig:
    DEFAULT_AVAILABLE = "online"
    DEFAULT_NOT_AVAILABLE = "offline"

    def __init__(self, topic: str, payload_available: str = DEFAULT_AVAILABLE, payload_not_available: str = DEFAULT_NOT_AVAILABLE):
        self.topic = validate_topic(topic)
        self.payload_available = payload_available
        self.payload_not_available = payload_not_available

    @property
    def topic(self):
        return self._topic

    @topic.setter
    def topic(self, topic: str):
        self._topic = validate_topic(topic)

    @property
    def payload_available(self):
        return self._payload_available

    @payload_available.setter
    def payload_available(self, payload_available: str):
        self._payload_available = validate_non_empty_string(payload_available, "TopicConfig 'payload_available'")

    @property
    def payload_not_available(self):
        return self._payload_not_available

    @payload_not_available.setter
    def payload_not_available(self, payload_not_available: str):
        self._payload_not_available = validate_non_empty_string(payload_not_available, "TopicConfig 'payload_not_available'")



from __future__ import annotations

from jhomeassistant.helper import validate_non_empty_string
from jhomeassistant.helper.abbreviations import OriginAbbreviation, Abbreviation


class Origin:
    def __init__(self):
        self.name = None
        self.sw_version = None
        self.url = None

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name: str | None):
        self._name = validate_non_empty_string(name, "Origin 'name'", True)

    @property
    def sw_version(self):
        return self._sw_version

    @sw_version.setter
    def sw_version(self, sw_version: str | None):
        self._sw_version = validate_non_empty_string(sw_version, "Origin 'sw_version'", True)

    @property
    def url(self):
        return self._url

    @url.setter
    def url(self, url: str | None):
        self._url = validate_non_empty_string(url, "Origin 'url'", True)

    def internal_to_dict(self):
        # Enforce required origin.name for device-based discovery
        if not isinstance(self.name, str) or self.name.strip() == "":
            raise ValueError(
                "origin.name is required for device-based discovery. "
                "Set HomeAssistantConnection.origin.name before discovery."
            )

        result = {
            OriginAbbreviation.NAME: self.name,
        }

        if self.sw_version:
            result[OriginAbbreviation.SW] = self.sw_version
        if self.url:
            result[OriginAbbreviation.URL] = self.url

        return {
            Abbreviation.ORIGIN: result
        }



from __future__ import annotations

import json
from typing import List, Union
from jmqtt import QualityOfService as QoS, MQTTMessage
from jmqtt.mqtt_connections import MQTTConnectionV3, MQTTConnectionV5

from jhomeassistant.features import Availability, Origin, TopicConfig
from jhomeassistant.helper import validate_discovery_prefix
from jhomeassistant.helper.abbreviations import resolve_abbreviation
from jhomeassistant.helper.scheduler import Scheduler, Schedule
from jhomeassistant.homeassistant_device import HomeAssistantDevice
from jhomeassistant.setup_logging import get_logger


logger = get_logger("HomeAssistantConnection")


def resolve_abbreviations(payload: dict, use_abbreviated_device_discovery: bool) -> dict:
    new_payload = {}
    for key, value in payload.items():
        if isinstance(value, dict):
            value = resolve_abbreviations(value, use_abbreviated_device_discovery)

        elif isinstance(value, list):
            normalized_list = []
            for item in value:
                if isinstance(item, dict):
                    normalized_list.append(
                        resolve_abbreviations(item, use_abbreviated_device_discovery)
                    )
                else:
                    normalized_list.append(item)
            value = normalized_list

        if not isinstance(key, str):
            key = resolve_abbreviation(key, use_abbreviated_device_discovery)

        new_payload[key] = value
    return new_payload


class HomeAssistantConnection:
    def __init__(self, connection: Union[MQTTConnectionV3, MQTTConnectionV5], use_abbreviated_device_discovery=False):
        self._use_abbreviated_device_discovery = use_abbreviated_device_discovery
        self._connection = connection
        self._devices: List[HomeAssistantDevice] = []
        self._discovery_prefix = 'homeassistant'

        self.origin = Origin()
        self.availability = Availability(self._connection.availability_topic)
        self.ha_status = TopicConfig("homeassistant/status")
        self.qos: QoS | None = None
        self.encoding: str | None = None

    def discovery_prefix(self, discovery_prefix: str) -> HomeAssistantConnection:
        self._discovery_prefix = validate_discovery_prefix(discovery_prefix)
        return self

    def add_devices(self, *devices: HomeAssistantDevice) -> HomeAssistantConnection:
        for device in devices:
            self._devices.append(device)
        return self

    def _discovery_gen(self):
        for device in self._devices:
            # Origin fallback
            if self.origin.name is None:
                logger.info(f"Using device name {device.name} as origin.name because no custom name was provided.")
                self.origin.name = device.name

            # Inherit connection-level QoS/encoding if set
            if device.qos is None and self.qos is not None:
                device.qos = self.qos
                logger.info(f"Inherit connection QoS={self.qos.name} to device {device.name}")
            if device.encoding is None and self.encoding is not None:
                device.encoding = self.encoding
                logger.info(f"Inherit connection encoding={self.encoding} to device {device.name}")

            device.availability.internal_merge(self.availability)
            discovery_topic, discovery_payload = device.internal_discovery(self._discovery_prefix)

            discovery_payload = {
                **discovery_payload,
                **self.origin.internal_to_dict()
            }

            yield discovery_topic, resolve_abbreviations(discovery_payload, self._use_abbreviated_device_discovery)

    def discovery_text(self):
        text = ""
        for topic, payload in self._discovery_gen():
            pretty = json.dumps(
                payload,
                indent=2,
                sort_keys=True,
                ensure_ascii=False,
            )
            text += f"-------------------------- Topic: {topic} --------------------------\n{pretty}\n\n"
        return text

    def _discovery(self, publish_timeout):
        """Publish all discovery payloads and wait for completion."""
        discovery_infos = []
        for discovery_topic, discovery_payload in self._discovery_gen():
            payload_str = json.dumps(discovery_payload, separators=(",", ":"))
            logger.info(f"Publishing discovery: topic={discovery_topic} retained=True qos={QoS.AtLeastOnce.name} bytes={len(payload_str)}")
            logger.debug(f"Discovery payload: {payload_str}")
            discovery_infos.append(self._connection.publish(discovery_topic, payload_str, QoS.AtLeastOnce, True))

        # wait for all publish to finish
        for i in discovery_infos:
            i.wait_for_publish(publish_timeout)

    def _entities(self):
        for device in self._devices:
            for entity in device.entities:
                yield entity

    def homeassistant_status(self, connection, client, userdata, message: MQTTMessage):
        for entity in self._entities():
            if message.text == self.ha_status.payload_available:
                entity.home_assistant_birth(connection)
            elif message.text == self.ha_status.payload_not_available:
                entity.home_assistant_death(connection)
            else:
                logger.warning(f"Unknown Home Assistant status(topic={self._discovery_prefix}/status) message {message}")

    def run(self, schedule_resolution: float = 1.0, publish_timeout: float | None = None):
        if not self._connection.is_connected:
            self._connection.connect()

        self._discovery(publish_timeout)

        # Subscribe HA Status
        self._connection.subscribe(self.ha_status.topic, self.homeassistant_status)

        # register publishers with interval
        tasks = [schedule for entity in self._entities() for schedule in entity.schedules]
        Scheduler(*tasks).run_forever(schedule_resolution, self._connection)



from re import match

from jhomeassistant.types.units.base import BaseUnit


class MonetaryUnit(BaseUnit):
    """Holds an ISO 4217 currency code like 'EUR' or 'USD'."""
    __slots__ = ("VALUE",)

    def __init__(self, code: str):
        # Example: "EUR", "USD", "JPY"
        super().__init__()
        if code is None or not match(r"^[A-Z]{3}$", code):
            raise ValueError(
                f"Invalid unit '{code}' for device_class 'monetary'. "
                f"Allowed: 3-letter ISO 4217 currency code. See https://en.wikipedia.org/wiki/ISO_4217#Active_codes"
            )
        self.VALUE = code

    def __str__(self) -> str:
        return self.VALUE

    def __repr__(self) -> str:
        return f"MonetaryUnit(VALUE={self.VALUE!r})"



from __future__ import annotations

import hashlib
import base64
import math
import secrets
from slugify import slugify

from jhomeassistant.types import Component


def ha_slugify(name: str) -> str:
    """ASCII transliterate, lowercase, allow [a-z0-9_], collapse/trim underscores."""
    s = slugify(name, lowercase=True, separator="_", regex_pattern=r"[^a-z0-9_]+")
    s = s.strip("_")
    return s


def get_default_entity_id(component: Component, device_name, entity_name):
    device_name, entity_name = ha_slugify(device_name), ha_slugify(entity_name)
    while len(f"{component.value}.{device_name}_{entity_name}") > 255 and len(device_name) > 20:
        device_name = device_name[:-1]
    return f"{component.value}.{device_name}_{entity_name}"[:255]


def build_identifier(seed: str, length: int = 32, namespace: str | None = None) -> str:
    """
    Deterministic, url-safe ID derived from 'seed' and optional 'namespace' (salt).
    Use for Home Assistant unique_id / device identifiers.

    - Output alphabet: [A-Za-z0-9-_]
    - Length: exact 'length'
    - If 'namespace' is provided, it is combined with the seed using a non-printable
      separator to avoid accidental collisions with user content.
    """
    if length < 1:
        raise ValueError("length must be >= 1")
    if len(seed.strip()) < 0 or (namespace is not None and len(namespace.strip()) < 0):
        raise ValueError("seed and namespace must be non-empty")

    if namespace is not None:
        # Use ASCII Unit Separator to avoid collisions with normal text concatenations.
        seed = f"{namespace}\x1f{seed}"

    seed_norm = seed.strip()
    # Ensure enough hash bytes to cover base64 url encoding.
    digest_size = max(16, math.ceil(length * 3 / 4))
    raw = hashlib.blake2s(seed_norm.encode("utf-8"), digest_size=digest_size).digest()
    enc = base64.urlsafe_b64encode(raw).rstrip(b"=").decode("ascii")
    return enc[:length]


"""def generate_rand_str(length: int = 16) -> str:
    # Uniform URL-safe string, 6 bits per char
    nbytes = (length * 6 + 7) // 8  # integer ceil(length*3/4)
    return base64.urlsafe_b64encode(secrets.token_bytes(nbytes)).rstrip(b"=").decode("ascii")[:length]"""



from jhomeassistant.types.units.base import BaseUnit


class PrecipitationIntensityUnit(BaseUnit):
    INCH_PER_DAY = "in/d"
    INCH_PER_HOUR = "in/h"
    MILLIMETER_PER_DAY = "mm/d"
    MILLIMETER_PER_HOUR = "mm/h"



from jhomeassistant.types.units.base import BaseUnit


class WindSpeedUnit(BaseUnit):
    FOOT_PER_SECOND = "ft/s"
    KILOMETER_PER_HOUR = "km/h"
    KNOT = "kn"
    METER_PER_SECOND = "m/s"
    MILE_PER_HOUR = "mph"
    BEAUFORT = "Beaufort"



from enum import StrEnum


class EntityCategory(StrEnum):
    """
    Entity categories supported by Home Assistant.

    - CONFIG: Entity configures a device (shows under "Settings"/"Configuration").
    - DIAGNOSTIC: Entity provides diagnostics/telemetry for a device.
    """
    CONFIG = "config"
    DIAGNOSTIC = "diagnostic"



from jhomeassistant.types.units.base import BaseUnit


class VolumeFlowRateSensorUnit(BaseUnit):
    CUBIC_METER_PER_HOUR = "m³/h"
    CUBIC_METER_PER_MINUTE = "m³/min"
    CUBIC_METER_PER_SECOND = "m³/s"
    CUBIC_FOOT_PER_MINUTE = "ft³/min"
    LITER_PER_HOUR = "L/h"
    LITER_PER_MINUTE = "L/min"
    LITER_PER_SECOND = "L/s"
    GALLON_PER_MINUTE = "gal/min"
    MILLILITER_PER_SECOND = "mL/s"
    GALLON_PER_HOUR = "gal/h"


class VolumeFlowRateNumberUnit(BaseUnit):
    CUBIC_METER_PER_HOUR = "m³/h"
    CUBIC_METER_PER_MINUTE = "m³/min"
    CUBIC_METER_PER_SECOND = "m³/s"
    CUBIC_FOOT_PER_MINUTE = "ft³/min"
    LITER_PER_HOUR = "L/h"
    LITER_PER_MINUTE = "L/min"
    LITER_PER_SECOND = "L/s"
    GALLON_PER_MINUTE = "gal/min"
    MILLILITER_PER_SECOND = "mL/s"



from jhomeassistant.types.device_classes.base_device_class import BaseDeviceClass


class ButtonDeviceClass(BaseDeviceClass):
    NONE = None
    IDENTIFY = "identify"
    RESTART = "restart"
    UPDATE = "update"



from jhomeassistant.types.units.base import BaseUnit


class DataRateUnit(BaseUnit):
    BIT_PER_SECOND = "bit/s"
    KILOBIT_PER_SECOND = "kbit/s"
    MEGABIT_PER_SECOND = "Mbit/s"
    GIGABIT_PER_SECOND = "Gbit/s"
    BYTE_PER_SECOND = "B/s"
    KILOBYTE_PER_SECOND = "kB/s"
    MEGABYTE_PER_SECOND = "MB/s"
    GIGABYTE_PER_SECOND = "GB/s"
    KIBIBYTE_PER_SECOND = "KiB/s"
    MEBIBYTE_PER_SECOND = "MiB/s"
    GIBIBYTE_PER_SECOND = "GiB/s"




from . import types
from . import entities
from .homeassistant_device import HomeAssistantDevice
from .homeassistant_connection import HomeAssistantConnection

__all__ = ["HomeAssistantDevice", "HomeAssistantConnection"]
__version__ = "0.1.0"



import time

from jhomeassistant.helper.scheduler import Schedule


class Scheduler:
    def __init__(self, *tasks: Schedule):
        self.tasks = list(tasks)

    def run_forever(self, tick_resolution: float, connection):
        while True:
            now = time.time()
            for t in self.tasks:
                t.run(now, connection)
            time.sleep(tick_resolution)



from jhomeassistant.types.device_classes.base_device_class import BaseDeviceClass


class BinarySensorDeviceClass(BaseDeviceClass):
    NONE = None
    BATTERY = "battery"
    BATTERY_CHARGING = "battery_charging"
    CARBON_MONOXIDE = "carbon_monoxide"
    COLD = "cold"
    CONNECTIVITY = "connectivity"
    DOOR = "door"
    GARAGE_DOOR = "garage_door"
    GAS = "gas"
    HEAT = "heat"
    LIGHT = "light"
    LOCK = "lock"
    MOISTURE = "moisture"
    MOTION = "motion"
    MOVING = "moving"
    OCCUPANCY = "occupancy"
    OPENING = "opening"
    PLUG = "plug"
    POWER = "power"
    PRESENCE = "presence"
    PROBLEM = "problem"
    RUNNING = "running"
    SAFETY = "safety"
    SMOKE = "smoke"
    SOUND = "sound"
    TAMPER = "tamper"
    UPDATE = "update"
    VIBRATION = "vibration"
    WINDOW = "window"



from __future__ import annotations
from typing import Callable, Union, List

from jmqtt import MQTTConnectionV3, MQTTConnectionV5

from jhomeassistant.features import Availability
from jhomeassistant.helper import validate_non_empty_string, build_identifier
from jhomeassistant.helper.abbreviations import Abbreviation
from jhomeassistant.helper.scheduler import Schedule
from jhomeassistant.types.component import Component


class HomeAssistantEntityBase:
    def __init__(self, component: Component, name):
        validate_non_empty_string(name, "Entity 'name'")
        self._schedules: List[Schedule] = []
        self._name = name
        self._component = component

        self.availability = Availability()

    def add_schedule(self, interval: float, function: Callable[[Union[MQTTConnectionV3, MQTTConnectionV5]], None]):
        self._schedules.append(Schedule(interval, function))
        return self

    @property
    def schedules(self):
        return self._schedules

    @property
    def identifier(self):
        return build_identifier(self._name)

    @property
    def name(self):
        return self._name

    @property
    def platform(self):
        return self._component

    @property
    def internal_discovery_payload(self):
        return {
            "component": self._component.value,
            Abbreviation.NAME: self._name,
            **self.availability.internal_to_dict()
        }

    def home_assistant_birth(self, connection: Union[MQTTConnectionV3, MQTTConnectionV5]):
        pass

    def home_assistant_death(self, connection: Union[MQTTConnectionV3, MQTTConnectionV5]):
        pass



from enum import Enum


class Abbreviation(Enum):
    ACTION_TOPIC = ("action_topic", "act_t")
    ACTION_TEMPLATE = ("action_template", "act_tpl")
    AUTOMATION_TYPE = ("automation_type", "atype")
    AUX_COMMAND_TOPIC = ("aux_command_topic", "aux_cmd_t")
    AUX_STATE_TOPIC = ("aux_state_topic", "aux_stat_t")
    AUX_STATE_TEMPLATE = ("aux_state_template", "aux_stat_tpl")
    AVAILABLE_TONES = ("available_tones", "av_tones")
    AVAILABILITY = ("availability", "avty")
    AVAILABILITY_MODE = ("availability_mode", "avty_mode")
    AVAILABILITY_TOPIC = ("availability_topic", "avty_t")
    AVAILABILITY_TEMPLATE = ("availability_template", "avty_tpl")
    AWAY_MODE_COMMAND_TOPIC = ("away_mode_command_topic", "away_mode_cmd_t")
    AWAY_MODE_STATE_TOPIC = ("away_mode_state_topic", "away_mode_stat_t")
    AWAY_MODE_STATE_TEMPLATE = ("away_mode_state_template", "away_mode_stat_tpl")
    BLUE_TEMPLATE = ("blue_template", "b_tpl")
    BRIGHTNESS_COMMAND_TOPIC = ("brightness_command_topic", "bri_cmd_t")
    BRIGHTNESS_COMMAND_TEMPLATE = ("brightness_command_template", "bri_cmd_tpl")
    BRIGHTNESS_SCALE = ("brightness_scale", "bri_scl")
    BRIGHTNESS_STATE_TOPIC = ("brightness_state_topic", "bri_stat_t")
    BRIGHTNESS_TEMPLATE = ("brightness_template", "bri_tpl")
    BRIGHTNESS_VALUE_TEMPLATE = ("brightness_value_template", "bri_val_tpl")
    COLOR_TEMP_COMMAND_TEMPLATE = ("color_temp_command_template", "clr_temp_cmd_tpl")
    COLOR_TEMP_COMMAND_TOPIC = ("color_temp_command_topic", "clr_temp_cmd_t")
    COLOR_TEMP_KELVIN = ("color_temp_kelvin", "clr_temp_k")
    COLOR_TEMP_STATE_TOPIC = ("color_temp_state_topic", "clr_temp_stat_t")
    COLOR_TEMP_TEMPLATE = ("color_temp_template", "clr_temp_tpl")
    COLOR_TEMP_VALUE_TEMPLATE = ("color_temp_value_template", "clr_temp_val_tpl")
    COLOR_MODE = ("color_mode", "clrm")
    COLOR_MODE_STATE_TOPIC = ("color_mode_state_topic", "clrm_stat_t")
    COLOR_MODE_VALUE_TEMPLATE = ("color_mode_value_template", "clrm_val_tpl")
    COMMAND_OFF_TEMPLATE = ("command_off_template", "cmd_off_tpl")
    COMMAND_ON_TEMPLATE = ("command_on_template", "cmd_on_tpl")
    COMMAND_TOPIC = ("command_topic", "cmd_t")
    COMMAND_TEMPLATE = ("command_template", "cmd_tpl")
    COMPONENTS = ("components", "cmps")
    CODE_ARM_REQUIRED = ("code_arm_required", "cod_arm_req")
    CODE_DISARM_REQUIRED = ("code_disarm_required", "cod_dis_req")
    CODE_TRIGGER_REQUIRED = ("code_trigger_required", "cod_trig_req")
    CONTENT_TYPE = ("content_type", "cont_type")
    CURRENT_TEMPERATURE_TOPIC = ("current_temperature_topic", "curr_temp_t")
    CURRENT_TEMPERATURE_TEMPLATE = ("current_temperature_template", "curr_temp_tpl")
    DEFAULT_ENTITY_ID = ("default_entity_id", "def_ent_id")
    DEVICE = ("device", "dev")
    DEVICE_CLASS = ("device_class", "dev_cla")
    DIRECTION_COMMAND_TOPIC = ("direction_command_topic", "dir_cmd_t")
    DIRECTION_COMMAND_TEMPLATE = ("direction_command_template", "dir_cmd_tpl")
    DIRECTION_STATE_TOPIC = ("direction_state_topic", "dir_stat_t")
    DIRECTION_VALUE_TEMPLATE = ("direction_value_template", "dir_val_tpl")
    DISPLAY_PRECISION = ("display_precision", "dsp_prc")
    ENCODING = ("encoding", "e")
    ENABLED_BY_DEFAULT = ("enabled_by_default", "en")
    ENTITY_CATEGORY = ("entity_category", "ent_cat")
    ENTITY_PICTURE = ("entity_picture", "ent_pic")
    EVENT_TYPES = ("event_types", "evt_typ")
    EXPIRE_AFTER = ("expire_after", "exp_aft")
    FAN_SPEED_LIST = ("fan_speed_list", "fanspd_lst")
    FLASH = ("flash", "flsh")
    FLASH_TIME_LONG = ("flash_time_long", "flsh_tlng")
    FLASH_TIME_SHORT = ("flash_time_short", "flsh_tsht")
    EFFECT_COMMAND_TOPIC = ("effect_command_topic", "fx_cmd_t")
    EFFECT_COMMAND_TEMPLATE = ("effect_command_template", "fx_cmd_tpl")
    EFFECT_LIST = ("effect_list", "fx_list")
    EFFECT_STATE_TOPIC = ("effect_state_topic", "fx_stat_t")
    EFFECT_TEMPLATE = ("effect_template", "fx_tpl")
    EFFECT_VALUE_TEMPLATE = ("effect_value_template", "fx_val_tpl")
    FAN_MODE_COMMAND_TOPIC = ("fan_mode_command_topic", "fan_mode_cmd_t")
    FAN_MODE_COMMAND_TEMPLATE = ("fan_mode_command_template", "fan_mode_cmd_tpl")
    FAN_MODE_STATE_TOPIC = ("fan_mode_state_topic", "fan_mode_stat_t")
    FAN_MODE_STATE_TEMPLATE = ("fan_mode_state_template", "fan_mode_stat_tpl")
    FORCE_UPDATE = ("force_update", "frc_upd")
    GREEN_TEMPLATE = ("green_template", "g_tpl")
    HS_COMMAND_TOPIC = ("hs_command_topic", "hs_cmd_t")
    HS_COMMAND_TEMPLATE = ("hs_command_template", "hs_cmd_tpl")
    HS_STATE_TOPIC = ("hs_state_topic", "hs_stat_t")
    HS_VALUE_TEMPLATE = ("hs_value_template", "hs_val_tpl")
    ICON = ("icon", "ic")
    IMAGE_ENCODING = ("image_encoding", "img_e")
    IMAGE_TOPIC = ("image_topic", "img_t")
    INITIAL = ("initial", "init")
    TARGET_HUMIDITY_COMMAND_TOPIC = ("target_humidity_command_topic", "hum_cmd_t")
    TARGET_HUMIDITY_COMMAND_TEMPLATE = ("target_humidity_command_template", "hum_cmd_tpl")
    TARGET_HUMIDITY_STATE_TOPIC = ("target_humidity_state_topic", "hum_stat_t")
    TARGET_HUMIDITY_STATE_TEMPLATE = ("target_humidity_state_template", "hum_state_tpl")
    JSON_ATTRIBUTES = ("json_attributes", "json_attr")
    JSON_ATTRIBUTES_TOPIC = ("json_attributes_topic", "json_attr_t")
    JSON_ATTRIBUTES_TEMPLATE = ("json_attributes_template", "json_attr_tpl")
    LATEST_VERSION_TOPIC = ("latest_version_topic", "l_ver_t")
    LATEST_VERSION_TEMPLATE = ("latest_version_template", "l_ver_tpl")
    LAST_RESET_TOPIC = ("last_reset_topic", "lrst_t")
    LAST_RESET_VALUE_TEMPLATE = ("last_reset_value_template", "lrst_val_tpl")
    MAX = ("max", "max")
    MAX_HUMIDITY = ("max_humidity", "max_hum")
    MAX_KELVIN = ("max_kelvin", "max_k")
    MAX_MIREDS = ("max_mireds", "max_mirs")
    MAX_TEMP = ("max_temp", "max_temp")
    MIGRATE_DISCOVERY = ("migrate_discovery", "migr_discvry")
    MIN = ("min", "min")
    MIN_HUMIDITY = ("min_humidity", "min_hum")
    MIN_KELVIN = ("min_kelvin", "min_k")
    MIN_MIREDS = ("min_mireds", "min_mirs")
    MIN_TEMP = ("min_temp", "min_temp")
    MODE = ("mode", "mode")
    MODE_COMMAND_TOPIC = ("mode_command_topic", "mode_cmd_t")
    MODE_COMMAND_TEMPLATE = ("mode_command_template", "mode_cmd_tpl")
    MODE_STATE_TOPIC = ("mode_state_topic", "mode_stat_t")
    MODE_STATE_TEMPLATE = ("mode_state_template", "mode_stat_tpl")
    MODES = ("modes", "modes")
    NAME = ("name", "name")
    ORIGIN = ("origin", "o")
    OFF_DELAY = ("off_delay", "off_dly")
    ON_COMMAND_TYPE = ("on_command_type", "on_cmd_type")
    OPTIONS = ("options", "ops")
    OPTIMISTIC = ("optimistic", "opt")
    OSCILLATION_COMMAND_TOPIC = ("oscillation_command_topic", "osc_cmd_t")
    OSCILLATION_COMMAND_TEMPLATE = ("oscillation_command_template", "osc_cmd_tpl")
    OSCILLATION_STATE_TOPIC = ("oscillation_state_topic", "osc_stat_t")
    OSCILLATION_VALUE_TEMPLATE = ("oscillation_value_template", "osc_val_tpl")
    PLATFORM = ("platform", "p")
    PERCENTAGE_COMMAND_TOPIC = ("percentage_command_topic", "pct_cmd_t")
    PERCENTAGE_COMMAND_TEMPLATE = ("percentage_command_template", "pct_cmd_tpl")
    PERCENTAGE_STATE_TOPIC = ("percentage_state_topic", "pct_stat_t")
    PERCENTAGE_VALUE_TEMPLATE = ("percentage_value_template", "pct_val_tpl")
    PAYLOAD = ("payload", "pl")
    PAYLOAD_ARM_AWAY = ("payload_arm_away", "pl_arm_away")
    PAYLOAD_ARM_CUSTOM_BYPASS = ("payload_arm_custom_bypass", "pl_arm_custom_b")
    PAYLOAD_ARM_HOME = ("payload_arm_home", "pl_arm_home")
    PAYLOAD_ARM_NIGHT = ("payload_arm_night", "pl_arm_nite")
    PAYLOAD_ARM_VACATION = ("payload_arm_vacation", "pl_arm_vacation")
    PAYLOAD_AVAILABLE = ("payload_available", "pl_avail")
    PAYLOAD_CLEAN_SPOT = ("payload_clean_spot", "pl_cln_sp")
    PAYLOAD_CLOSE = ("payload_close", "pl_cls")
    PAYLOAD_DIRECTION_FORWARD = ("payload_direction_forward", "pl_dir_fwd")
    PAYLOAD_DIRECTION_REVERSE = ("payload_direction_reverse", "pl_dir_rev")
    PAYLOAD_DISARM = ("payload_disarm", "pl_disarm")
    PAYLOAD_HOME = ("payload_home", "pl_home")
    PAYLOAD_INSTALL = ("payload_install", "pl_inst")
    PAYLOAD_LOCATE = ("payload_locate", "pl_loc")
    PAYLOAD_LOCK = ("payload_lock", "pl_lock")
    PAYLOAD_NOT_AVAILABLE = ("payload_not_available", "pl_not_avail")
    PAYLOAD_NOT_HOME = ("payload_not_home", "pl_not_home")
    PAYLOAD_OFF = ("payload_off", "pl_off")
    PAYLOAD_ON = ("payload_on", "pl_on")
    PAYLOAD_OPEN = ("payload_open", "pl_open")
    PAYLOAD_OSCILLATION_OFF = ("payload_oscillation_off", "pl_osc_off")
    PAYLOAD_OSCILLATION_ON = ("payload_oscillation_on", "pl_osc_on")
    PAYLOAD_PAUSE = ("payload_pause", "pl_paus")
    PAYLOAD_STOP = ("payload_stop", "pl_stop")
    PAYLOAD_START = ("payload_start", "pl_strt")
    PAYLOAD_PRESS = ("payload_press", "pl_prs")
    PAYLOAD_RETURN_TO_BASE = ("payload_return_to_base", "pl_ret")
    PAYLOAD_RESET = ("payload_reset", "pl_rst")
    PAYLOAD_RESET_HUMIDITY = ("payload_reset_humidity", "pl_rst_hum")
    PAYLOAD_RESET_MODE = ("payload_reset_mode", "pl_rst_mode")
    PAYLOAD_RESET_PERCENTAGE = ("payload_reset_percentage", "pl_rst_pct")
    PAYLOAD_RESET_PRESET_MODE = ("payload_reset_preset_mode", "pl_rst_pr_mode")
    PAYLOAD_STOP_TILT = ("payload_stop_tilt", "pl_stop_tilt")
    PAYLOAD_START_PAUSE = ("payload_start_pause", "pl_stpa")
    PAYLOAD_TURN_OFF = ("payload_turn_off", "pl_toff")
    PAYLOAD_TURN_ON = ("payload_turn_on", "pl_ton")
    PAYLOAD_TRIGGER = ("payload_trigger", "pl_trig")
    PAYLOAD_UNLOCK = ("payload_unlock", "pl_unlk")
    REPORTS_POSITION = ("reports_position", "pos")
    POSITION_CLOSED = ("position_closed", "pos_clsd")
    POSITION_OPEN = ("position_open", "pos_open")
    PRESET_MODE_COMMAND_TOPIC = ("preset_mode_command_topic", "pr_mode_cmd_t")
    PRESET_MODE_COMMAND_TEMPLATE = ("preset_mode_command_template", "pr_mode_cmd_tpl")
    PRESET_MODE_STATE_TOPIC = ("preset_mode_state_topic", "pr_mode_stat_t")
    PRESET_MODE_VALUE_TEMPLATE = ("preset_mode_value_template", "pr_mode_val_tpl")
    PRESET_MODES = ("preset_modes", "pr_modes")
    PATTERN = ("pattern", "ptrn")
    RED_TEMPLATE = ("red_template", "r_tpl")
    RELEASE_SUMMARY = ("release_summary", "rel_s")
    RELEASE_URL = ("release_url", "rel_u")
    RETAIN = ("retain", "ret")
    RGB_COMMAND_TOPIC = ("rgb_command_topic", "rgb_cmd_t")
    RGB_COMMAND_TEMPLATE = ("rgb_command_template", "rgb_cmd_tpl")
    RGB_STATE_TOPIC = ("rgb_state_topic", "rgb_stat_t")
    RGB_VALUE_TEMPLATE = ("rgb_value_template", "rgb_val_tpl")
    RGBW_COMMAND_TOPIC = ("rgbw_command_topic", "rgbw_cmd_t")
    RGBW_COMMAND_TEMPLATE = ("rgbw_command_template", "rgbw_cmd_tpl")
    RGBW_STATE_TOPIC = ("rgbw_state_topic", "rgbw_stat_t")
    RGBW_VALUE_TEMPLATE = ("rgbw_value_template", "rgbw_val_tpl")
    RGBWW_COMMAND_TOPIC = ("rgbww_command_topic", "rgbww_cmd_t")
    RGBWW_COMMAND_TEMPLATE = ("rgbww_command_template", "rgbww_cmd_tpl")
    RGBWW_STATE_TOPIC = ("rgbww_state_topic", "rgbww_stat_t")
    RGBWW_VALUE_TEMPLATE = ("rgbww_value_template", "rgbww_val_tpl")
    SEND_COMMAND_TOPIC = ("send_command_topic", "send_cmd_t")
    SEND_IF_OFF = ("send_if_off", "send_if_off")
    SET_FAN_SPEED_TOPIC = ("set_fan_speed_topic", "set_fan_spd_t")
    SET_POSITION_TOPIC = ("set_position_topic", "set_pos_t")
    SET_POSITION_TEMPLATE = ("set_position_template", "set_pos_tpl")
    POSITION_TOPIC = ("position_topic", "pos_t")
    POSITION_TEMPLATE = ("position_template", "pos_tpl")
    SPEED_RANGE_MIN = ("speed_range_min", "spd_rng_min")
    SPEED_RANGE_MAX = ("speed_range_max", "spd_rng_max")
    SOURCE_TYPE = ("source_type", "src_type")
    STATE_CLASS = ("state_class", "stat_cla")
    STATE_CLOSING = ("state_closing", "stat_closing")
    STATE_CLOSED = ("state_closed", "stat_clsd")
    STATE_JAMMED = ("state_jammed", "stat_jam")
    STATE_LOCKED = ("state_locked", "stat_locked")
    STATE_LOCKING = ("state_locking", "stat_locking")
    STATE_OFF = ("state_off", "stat_off")
    STATE_ON = ("state_on", "stat_on")
    STATE_OPEN = ("state_open", "stat_open")
    STATE_OPENING = ("state_opening", "stat_opening")
    STATE_STOPPED = ("state_stopped", "stat_stopped")
    STATE_UNLOCKED = ("state_unlocked", "stat_unlocked")
    STATE_UNLOCKING = ("state_unlocking", "stat_unlocking")
    STATE_TOPIC = ("state_topic", "stat_t")
    STATE_TEMPLATE = ("state_template", "stat_tpl")
    STATE_VALUE_TEMPLATE = ("state_value_template", "stat_val_tpl")
    STEP = ("step", "step")
    SUBTYPE = ("subtype", "stype")
    SUGGESTED_DISPLAY_PRECISION = ("suggested_display_precision", "sug_dsp_prc")
    SUPPORTED_COLOR_MODES = ("supported_color_modes", "sup_clrm")
    SUPPORT_DURATION = ("support_duration", "sup_dur")
    SUPPORT_VOLUME_SET = ("support_volume_set", "sup_vol")
    SUPPORTED_FEATURES = ("supported_features", "sup_feat")
    SWING_MODE_COMMAND_TOPIC = ("swing_mode_command_topic", "swing_mode_cmd_t")
    SWING_MODE_COMMAND_TEMPLATE = ("swing_mode_command_template", "swing_mode_cmd_tpl")
    SWING_MODE_STATE_TOPIC = ("swing_mode_state_topic", "swing_mode_stat_t")
    SWING_MODE_STATE_TEMPLATE = ("swing_mode_state_template", "swing_mode_stat_tpl")
    TOPIC = ("topic", "t")
    TEMPERATURE_COMMAND_TOPIC = ("temperature_command_topic", "temp_cmd_t")
    TEMPERATURE_COMMAND_TEMPLATE = ("temperature_command_template", "temp_cmd_tpl")
    TEMPERATURE_HIGH_COMMAND_TOPIC = ("temperature_high_command_topic", "temp_hi_cmd_t")
    TEMPERATURE_HIGH_COMMAND_TEMPLATE = ("temperature_high_command_template", "temp_hi_cmd_tpl")
    TEMPERATURE_HIGH_STATE_TOPIC = ("temperature_high_state_topic", "temp_hi_stat_t")
    TEMPERATURE_HIGH_STATE_TEMPLATE = ("temperature_high_state_template", "temp_hi_stat_tpl")
    TEMPERATURE_LOW_COMMAND_TOPIC = ("temperature_low_command_topic", "temp_lo_cmd_t")
    TEMPERATURE_LOW_COMMAND_TEMPLATE = ("temperature_low_command_template", "temp_lo_cmd_tpl")
    TEMPERATURE_LOW_STATE_TOPIC = ("temperature_low_state_topic", "temp_lo_stat_t")
    TEMPERATURE_LOW_STATE_TEMPLATE = ("temperature_low_state_template", "temp_lo_stat_tpl")
    TEMPERATURE_STATE_TOPIC = ("temperature_state_topic", "temp_stat_t")
    TEMPERATURE_STATE_TEMPLATE = ("temperature_state_template", "temp_stat_tpl")
    TEMPERATURE_UNIT = ("temperature_unit", "temp_unit")
    TILT_CLOSED_VALUE = ("tilt_closed_value", "tilt_clsd_val")
    TILT_COMMAND_TOPIC = ("tilt_command_topic", "tilt_cmd_t")
    TILT_COMMAND_TEMPLATE = ("tilt_command_template", "tilt_cmd_tpl")
    TILT_MAX = ("tilt_max", "tilt_max")
    TILT_MIN = ("tilt_min", "tilt_min")
    TILT_OPENED_VALUE = ("tilt_opened_value", "tilt_opnd_val")
    TILT_OPTIMISTIC = ("tilt_optimistic", "tilt_opt")
    TILT_STATUS_TOPIC = ("tilt_status_topic", "tilt_status_t")
    TILT_STATUS_TEMPLATE = ("tilt_status_template", "tilt_status_tpl")
    TITLE = ("title", "tit")
    TRANSITION = ("transition", "trns")
    UNIQUE_ID = ("unique_id", "uniq_id")
    UNIT_OF_MEASUREMENT = ("unit_of_measurement", "unit_of_meas")
    URL_TOPIC = ("url_topic", "url_t")
    URL_TEMPLATE = ("url_template", "url_tpl")
    VALUE_TEMPLATE = ("value_template", "val_tpl")
    WHITE_COMMAND_TOPIC = ("white_command_topic", "whit_cmd_t")
    WHITE_SCALE = ("white_scale", "whit_scl")
    XY_COMMAND_TOPIC = ("xy_command_topic", "xy_cmd_t")
    XY_COMMAND_TEMPLATE = ("xy_command_template", "xy_cmd_tpl")
    XY_STATE_TOPIC = ("xy_state_topic", "xy_stat_t")
    XY_VALUE_TEMPLATE = ("xy_value_template", "xy_val_tpl")


from __future__ import annotations

from jhomeassistant.features import TopicConfig
from jhomeassistant.helper import validate_non_empty_string
from jhomeassistant.helper.abbreviations import Abbreviation


class AvailabilityItem(TopicConfig):
    def __init__(self, topic: str, payload_available: str = TopicConfig.DEFAULT_AVAILABLE,
                 payload_not_available: str = TopicConfig.DEFAULT_NOT_AVAILABLE, value_template=None):
        super().__init__(topic, payload_available, payload_not_available)
        self.value_template = value_template

    @property
    def value_template(self):
        return self._value_template

    @value_template.setter
    def value_template(self, value_template: str | None):
        self._value_template = validate_non_empty_string(value_template, "Origin 'value_template'", True)

    def internal_to_dict(self):
        result = {Abbreviation.TOPIC: self.topic}

        if self.payload_available != TopicConfig.DEFAULT_AVAILABLE:
            result[Abbreviation.PAYLOAD_AVAILABLE] = self.payload_available
        if self.payload_not_available != TopicConfig.DEFAULT_NOT_AVAILABLE:
            result[Abbreviation.PAYLOAD_NOT_AVAILABLE] = self.payload_not_available
        if self.value_template is not None:
            result[Abbreviation.VALUE_TEMPLATE] = self.value_template

        return result

    def __repr__(self):
        return f"AvailabilityItem(topic={self.topic}, payload_available={self.payload_available}, payload_not_available={self.payload_not_available}, value_template={self.value_template})"



from .absolute_humidity_units import AbsoluteHumidityUnit
from .apparent_power_units import ApparentPowerUnit
from .area_units import AreaUnit
from .atmospheric_pressure_units import AtmosphericPressureUnit
from .battery_units import BatteryUnit
from .blood_glucose_concentration_units import BloodGlucoseConcentrationUnit
from .carbon_dioxide_units import CarbonDioxideUnit
from .carbon_monoxide_units import CarbonMonoxideNumberUnit, CarbonMonoxideSensorUnit
from .conductivity_units import ConductivityUnit
from .current_units import CurrentUnit
from .data_rate_units import DataRateUnit
from .data_size_units import DataSizeUnit
from .distance_units import DistanceUnit
from .duration_units import DurationUnit
from .energy_units import EnergyUnit
from .energy_distance_units import EnergyDistanceUnit
from .energy_storage_units import EnergyStorageUnit
from .frequency_units import FrequencyUnit
from .gas_units import GasUnit
from .humidity_units import HumidityUnit
from .illuminance_units import IlluminanceUnit
from .irradiance_units import IrradianceUnit
from .moisture_units import MoistureUnit
from .monetary_units import MonetaryUnit
from .nitrogen_dioxide_units import NitrogenDioxideUnit
from .nitrogen_monoxide_units import NitrogenMonoxideUnit
from .nitrous_oxide_units import NitrousOxideUnit
from .ozone_units import OzoneUnit
from .pm1_units import Pm1Unit
from .pm25_units import Pm25Unit
from .pm4_units import Pm4Unit
from .pm10_units import Pm10Unit
from .power_units import PowerUnit
from .power_factor_units import PowerFactorUnit  # subclass of NoUnit
from .precipitation_units import PrecipitationUnit
from .precipitation_intensity_units import PrecipitationIntensityUnit
from .pressure_units import PressureSensorUnit, PressureNumberUnit
from .reactive_energy_units import ReactiveEnergyUnit
from .reactive_power_units import ReactivePowerUnit
from .signal_strength_units import SignalStrengthUnit
from .sound_pressure_units import SoundPressureUnit
from .speed_units import SpeedUnit
from .sulphur_dioxide_units import SulphurDioxideUnit
from .temperature_units import TemperatureUnit
from .temperature_delta_units import TemperatureDeltaUnit
from .volatile_organic_compounds_units import VolatileOrganicCompoundsUnit
from .volatile_organic_compounds_parts_units import VolatileOrganicCompoundsPartsUnit
from .voltage_units import VoltageUnit
from .volume_units import VolumeUnit
from .volume_flow_rate_units import VolumeFlowRateSensorUnit, VolumeFlowRateNumberUnit
from .volume_storage_units import VolumeStorageUnit
from .water_units import WaterUnit
from .weight_units import WeightUnit
from .wind_direction_units import WindDirectionUnit
from .wind_speed_units import WindSpeedUnit

__all__ = [
    "AbsoluteHumidityUnit",
    "ApparentPowerUnit",
    "AreaUnit",
    "AtmosphericPressureUnit",
    "BatteryUnit",
    "BloodGlucoseConcentrationUnit",
    "CarbonDioxideUnit",
    "CarbonMonoxideNumberUnit",
    "CarbonMonoxideSensorUnit",
    "ConductivityUnit",
    "CurrentUnit",
    "DataRateUnit",
    "DataSizeUnit",
    "DistanceUnit",
    "DurationUnit",
    "EnergyUnit",
    "EnergyDistanceUnit",
    "EnergyStorageUnit",
    "FrequencyUnit",
    "GasUnit",
    "HumidityUnit",
    "IlluminanceUnit",
    "IrradianceUnit",
    "MoistureUnit",
    "MonetaryUnit",
    "NitrogenDioxideUnit",
    "NitrogenMonoxideUnit",
    "NitrousOxideUnit",
    "OzoneUnit",
    "Pm1Unit",
    "Pm25Unit",
    "Pm4Unit",
    "Pm10Unit",
    "PowerUnit",
    "PowerFactorUnit",
    "PrecipitationUnit",
    "PrecipitationIntensityUnit",
    "PressureSensorUnit",
    "PressureNumberUnit",
    "ReactiveEnergyUnit",
    "ReactivePowerUnit",
    "SignalStrengthUnit",
    "SoundPressureUnit",
    "SpeedUnit",
    "SulphurDioxideUnit",
    "TemperatureUnit",
    "TemperatureDeltaUnit",
    "VolatileOrganicCompoundsUnit",
    "VolatileOrganicCompoundsPartsUnit",
    "VoltageUnit",
    "VolumeUnit",
    "VolumeFlowRateSensorUnit",
    "VolumeFlowRateNumberUnit",
    "VolumeStorageUnit",
    "WaterUnit",
    "WeightUnit",
    "WindDirectionUnit",
    "WindSpeedUnit"
]


from .validation import (validate_discovery_prefix, validate_entity_specification, validate_icon,
                         validate_topic, validate_non_empty_string)
from .device_data_detection import collect_ha_device_facts
from .naming import get_default_entity_id, build_identifier
